# -*- coding: utf-8 -*-
"""
Support for AIS AudioBooks

For more details about this component, please refer to the documentation at
https://ai-speaker.com
"""
import asyncio
import logging
import requests
import json
import os.path
from homeassistant.components import ais_cloud
from homeassistant.ais_dom import ais_global
aisCloud = ais_cloud.AisCloudWS()

DOMAIN = 'ais_audiobooks_service'
PERSISTENCE_AUDIOBOOKS = '/.dom/audiobboks.json'
_LOGGER = logging.getLogger(__name__)
AUDIOBOOKS_WS_URL = 'https://wolnelektury.pl/api/audiobooks/?format=json'


@asyncio.coroutine
def async_setup(hass, config):
    """Register the service."""
    config = config.get(DOMAIN, {})

    _LOGGER.info("Initialize the authors list.")
    data = hass.data[DOMAIN] = AudioBooksData(hass, config)
    yield from data.async_load_all_books()

    # register services
    def get_books(call):
        _LOGGER.info("get_books")
        data.get_books(call)

    def get_chapters(call):
        _LOGGER.info("get_chapters")
        data.get_chapters(call)

    def select_chapter(call):
        _LOGGER.info("select_chapter")
        data.select_chapter(call)

    hass.services.async_register(
        DOMAIN, 'get_books', get_books)
    hass.services.async_register(
        DOMAIN, 'get_chapters', get_chapters)
    hass.services.async_register(
        DOMAIN, 'select_chapter', select_chapter)

    return True


class AudioBooksData:
    """Class to hold audiobooks data."""

    def __init__(self, hass, config):
        """Initialize the books authors."""
        self.hass = hass
        self.all_books = []

    def get_books(self, call):
        """Load books for the selected author."""
        if "author" not in call.data:
            _LOGGER.error("No author")
            return []

        if call.data["author"] == ais_global.G_EMPTY_OPTION:
            # reset status for item below
            self.hass.states.async_set("sensor.audiobookslist", -1, {})
            self.hass.states.async_set("sensor.audiobookschapterslist", -1, {})
            return

        list_info = {}
        list_idx = 0
        for item in self.all_books:
            if item["author"] == call.data["author"]:
                list_info[list_idx] = {}
                try:
                    list_info[list_idx]["thumbnail"] = "https://wolnelektury.pl/media/" + item["cover_thumb"]
                except Exception:
                    list_info[list_idx]["thumbnail"] = item["simple_thumb"]
                list_info[list_idx]["title"] = item["title"]
                list_info[list_idx]["name"] = item["title"]
                list_info[list_idx]["uri"] = item["url"]
                list_info[list_idx]["media_source"] = ais_global.G_AN_AUDIOBOOK
                list_info[list_idx]["audio_type"] = ais_global.G_AN_AUDIOBOOK
                list_info[list_idx]["icon"] = "mdi:book-play"
                list_info[list_idx]["lookup_url"] = item["href"]
                list_info[list_idx]["lookup_name"] = item["title"]
                list_idx = list_idx + 1

        self.hass.states.async_set("sensor.audiobookslist", -1, list_info)
        import homeassistant.components.ais_ai_service as ais_ai
        if ais_ai.CURR_ENTITIE == 'input_select.book_autor':
            ais_ai.set_curr_entity(self.hass, 'sensor.audiobookslist')
            self.hass.services.call(
                'ais_ai_service', 'say_it', {"text": "Mamy " + str(len(list_info)) + " , wybierz książkę"})

    def get_chapters(self, call):
        """Load chapters for the selected book."""
        if "id" not in call.data:
            _LOGGER.error("No book id")
            return

        state = self.hass.states.get("sensor.audiobookslist")
        attr = state.attributes
        track = attr.get(int(call.data['id']))
        self.hass.states.async_set("sensor.audiobookslist", call.data['id'], attr)

        try:
            ws_resp = requests.get(track["lookup_url"] + "?format=json", timeout=10)
            data = ws_resp.json()

        except Exception as e:
            _LOGGER.error("Can't load chapters: " + str(e))
            self.hass.services.call('ais_ai_service', 'say_it', {"text": "Nie można pobrać rozdziałów"})
            return

        list_info = {}
        list_idx = 0
        for item in data["media"]:
            if item["type"] == "ogg":
                list_info[list_idx] = {}
                try:
                    list_info[list_idx]["thumbnail"] = data["cover"]
                except Exception:
                    list_info[list_idx]["thumbnail"] = data["simple_cover"]
                list_info[list_idx]["title"] = item["name"]
                list_info[list_idx]["name"] = item["name"]
                list_info[list_idx]["uri"] = item["url"]
                list_info[list_idx]["media_source"] = ais_global.G_AN_AUDIOBOOK_CHAPTER
                list_info[list_idx]["audio_type"] = ais_global.G_AN_AUDIOBOOK_CHAPTER
                list_info[list_idx]["icon"] = 'mdi:play'
                list_idx = list_idx + 1

        self.hass.states.async_set("sensor.audiobookschapterslist", 0, list_info)
        self.hass.services.call('ais_audiobooks_service', 'select_chapter', {"id": 0})

        # check if the change was done form remote
        import homeassistant.components.ais_ai_service as ais_ai
        if ais_ai.CURR_ENTITIE == 'sensor.audiobookslist':
            ais_ai.set_curr_entity(self.hass, 'sensor.audiobookschapterslist')
            if len(list_info) > 0:
                self.hass.services.call(
                    'ais_ai_service', 'say_it', {"text": "Włączam pierwszy rozdział z " + str(len(list_info))})
            else:
                self.hass.services.call(
                    'ais_ai_service', 'say_it', {"text": "Odtwarzam " + str(len(list_info))})

    def select_chapter(self, call):
        """Get chapter stream url for the selected name."""
        if "id" not in call.data:
            _LOGGER.error("No book chapter id")
            return

        state = self.hass.states.get("sensor.audiobookschapterslist")
        attr = state.attributes
        track = attr.get(int(call.data['id']))
        self.hass.states.async_set("sensor.audiobookschapterslist", call.data['id'], attr)

        # set stream uri, image and title
        _audio_info = json.dumps(
            {"IMAGE_URL": track["thumbnail"], "NAME": track["title"], "MEDIA_SOURCE": ais_global.G_AN_AUDIOBOOK,
             "media_content_id": track["uri"]})
        self.hass.services.call('media_player', 'play_media',
                                {"entity_id": ais_global.G_LOCAL_EXO_PLAYER_ENTITY_ID,
                                 "media_content_type": "ais_content_info",
                                 "media_content_id": _audio_info})

    @asyncio.coroutine
    def async_load_all_books(self):
        """Load all the books and cache the JSON."""

        def load():
            """Load the items synchronously."""
            self.hass.services.call(
                'ais_bookmarks', 'get_favorites', {"audio_source": ais_global.G_AN_AUDIOBOOK})

            path = self.hass.config.path() + PERSISTENCE_AUDIOBOOKS
            try:
                ws_resp = requests.get(AUDIOBOOKS_WS_URL, timeout=10)
                data = ws_resp.json()
                with open(path, 'w+') as my_file:
                    json.dump(data, my_file)
            except Exception as e:
                _LOGGER.info("Can't load books list: " + str(e))

            if not os.path.isfile(path):
                return

            with open(path) as file:
                self.all_books = json.loads(file.read())

            #  TODO data validation
            # self.all_books =
            # for book in books:
            #     b = {}
            #     self.all_books.append(b)

            authors = [ais_global.G_FAVORITE_OPTION]
            for item in self.all_books:
                if item["author"] not in authors:
                    authors.append(item["author"])
            self.hass.services.call(
                'input_select',
                'set_options', {
                    "entity_id": "input_select.book_autor",
                    "options": authors})

        yield from self.hass.async_add_job(load)



"""
Search the audio on YT
"""
import asyncio
import logging
import requests
import json
from homeassistant.components import ais_cloud
from homeassistant.ais_dom import ais_global
aisCloud = ais_cloud.AisCloudWS()
URL_BASE = 'https://www.googleapis.com/youtube/v3/search'
DEFAULT_ACTION = 'No video'

DOMAIN = 'ais_yt_service'
SERVICE_SEARCH = 'search'
ATTR_QUERY = 'query'
ATTR_NAME = 'name'

G_YT_KEY = None
_LOGGER = logging.getLogger(__name__)


@asyncio.coroutine
def async_setup(hass, config):
    """Register the service."""
    config = config.get(DOMAIN, {})
    data = hass.data[DOMAIN] = YouTubeData(hass)
    yield from data.get_key_async()

    @asyncio.coroutine
    def search(service):
        """search service about audio"""
        _LOGGER.debug('search')
        yield from data.process_search_async(service)

    # @asyncio.coroutine
    def select_track_uri(service):
        """select track uri"""
        _LOGGER.debug('select_track_uri')
        data.process_select_track_uri(service)

    # register services
    hass.services.async_register(DOMAIN, SERVICE_SEARCH, search)
    hass.services.async_register(DOMAIN, 'select_track_uri', select_track_uri)

    return True


class YouTubeData:
    """Class to hold YT data."""

    def __init__(self, hass):
        """Initialize the radio stations."""
        self.hass = hass

    @asyncio.coroutine
    def get_key_async(self):
        def load():
            global G_YT_KEY
            try:
                ws_resp = aisCloud.key("ytsearch")
                json_ws_resp = ws_resp.json()
                G_YT_KEY = json_ws_resp["key"]
            except:
                ais_global.G_OFFLINE_MODE = True

        yield from self.hass.async_add_job(load)

    @asyncio.coroutine
    def get_new_key(self, old_key, query, prev_page_token, next_page_token):
        global G_YT_KEY
        try:
            ws_resp = aisCloud.new_key("ytsearch", old_key)
            json_ws_resp = ws_resp.json()
            G_YT_KEY = json_ws_resp["key"]
        except Exception as e:
            _LOGGER.error("YouTube " + str(e))

        # check if new key is new (different from old one)
        if G_YT_KEY != old_key:
            # call the service again
            yield from self.hass.services.async_call(
                'ais_yt_service', 'search', {
                    ATTR_QUERY: query, 'prevPageToken': prev_page_token, 'nextPageToken': next_page_token})

    @asyncio.coroutine
    def process_search_async(self, call):
        """Search in service."""
        global G_YT_KEY
        query = None
        prev_page_token = None
        next_page_token = None
        text = None
        if ATTR_QUERY in call.data:
            query = call.data[ATTR_QUERY]

        if 'prevPageToken' in call.data:
            if call.data['prevPageToken'] is not None:
                prev_page_token = call.data['prevPageToken'].replace('prevPageToken_', '')

        if 'nextPageToken' in call.data:
            if call.data['nextPageToken'] is not None:
                next_page_token = call.data['nextPageToken'].replace('nextPageToken_', '')

        if (query is None or len(query.strip()) == 0) and prev_page_token is None and next_page_token is None:
            # get tracks from favorites
            yield from self.hass.services.async_call(
                'ais_bookmarks', 'get_favorites', {"audio_source": ais_global.G_AN_MUSIC})
            return

        if G_YT_KEY is None:
            try:
                ws_resp = aisCloud.key("ytsearch")
                json_ws_resp = ws_resp.json()
                G_YT_KEY = json_ws_resp["key"]
            except Exception as e:
                ais_global.G_OFFLINE_MODE = True
                yield from self.hass.services.async_call(
                    'ais_ai_service', 'say_it', {
                        "text": "Brak odpowiedzi, sprawdź połączenie z Intenetem"
                    })
                _LOGGER.error('process_search_async ' + str(e))
                return

        params = dict(order='relevance',
                      part='snippet',
                      key=G_YT_KEY,
                      maxResults=10,
                      type='video',
                      fields='items/id/videoId, items/snippet/title, items/snippet/thumbnails/medium/url, '
                             'pageInfo/totalResults, nextPageToken, prevPageToken')
        params.update({'q': query})
        if prev_page_token is not None:
            params.update({'pageToken': prev_page_token})
        if next_page_token is not None:
            params.update({'pageToken': next_page_token})
        data = requests.get(URL_BASE, params=params).json()

        # check if error
        if 'error' in data:
            for error in data['error']['errors']:
                if error['reason'] == 'quotaExceeded':
                    # get the new token and try again
                    yield from self.get_new_key(G_YT_KEY, query, prev_page_token, next_page_token)
                    return

                if text is None:
                    text = error['message']
                else:
                    text = text + ' ' + error['message']
            yield from self.hass.services.async_call('ais_ai_service', 'say_it', {"text": text})
            return

        list_info = {}
        list_idx = 0

        total_results = 0
        if 'pageInfo' in data:
            total_results = data['pageInfo']['totalResults']

        if 'prevPageToken' in data:
            list_info[list_idx] = {}
            list_info[list_idx]["title"] = "pobierz poprzednią stronę wyników wyszukiwania"
            list_info[list_idx]["name"] = "pobierz poprzednią stronę wyników wyszukiwania"
            list_info[list_idx]["thumbnail"] = "/static/icons/favicon-100x100.png"
            list_info[list_idx]["uri"] = 'prevPageToken_' + data['prevPageToken']
            list_info[list_idx]["media_source"] = ais_global.G_AN_MUSIC
            list_info[list_idx]["audio_type"] = ais_global.G_AN_MUSIC
            list_info[list_idx]["icon"] = 'mdi:page-previous'
            list_info[list_idx]["query"] = query
            list_idx = list_idx + 1

        if 'items' in data:
            for item in data['items']:
                list_info[list_idx] = {}
                list_info[list_idx]["title"] = item['snippet']['title']
                list_info[list_idx]["name"] = item['snippet']['title']
                # item['snippet']['description']
                list_info[list_idx]["thumbnail"] = item['snippet']['thumbnails']['medium']['url']
                list_info[list_idx]["uri"] = item['id']['videoId']
                list_info[list_idx]["media_source"] = ais_global.G_AN_MUSIC
                list_info[list_idx]["audio_type"] = ais_global.G_AN_MUSIC
                list_info[list_idx]["icon"] = 'mdi:play'
                list_idx = list_idx + 1

        if 'nextPageToken' in data:
            list_info[list_idx] = {}
            list_info[list_idx]["title"] = "pobierz następną stronę wyników wyszukiwania"
            list_info[list_idx]["name"] = "pobierz następną stronę wyników wyszukiwania"
            list_info[list_idx]["thumbnail"] = "/static/icons/favicon-100x100.png"
            list_info[list_idx]["uri"] = 'nextPageToken_' + data['nextPageToken']
            list_info[list_idx]["media_source"] = ais_global.G_AN_MUSIC
            list_info[list_idx]["audio_type"] = ais_global.G_AN_MUSIC
            list_info[list_idx]["icon"] = 'mdi:page-next'
            list_info[list_idx]["query"] = query

        # update list
        self.hass.states.async_set("sensor.youtubelist", -1, list_info)
        if len(list_info) > 0:
            # from remote
            import homeassistant.components.ais_ai_service as ais_ai
            if ais_ai.CURR_ENTITIE == 'input_text.ais_music_query' and ais_ai.CURR_BUTTON_CODE == 4:
                ais_ai.set_curr_entity(self.hass, 'sensor.youtubelist')
                ais_ai.CURR_ENTITIE_ENTERED = True
                text = "Znaleziono: %s, wybierz pozycję którą mam włączyć" % (str(total_results))
            else:
                if next_page_token is None and prev_page_token is None:
                    text = "Znaleziono: %s, włączam pierwszy: %s" % (str(total_results), list_info[0]["title"])
                    yield from self.hass.services.async_call('ais_yt_service', 'select_track_uri', {"id": 0})

                if ais_ai.CURR_ENTITIE == 'media_player.wbudowany_glosnik':
                    if next_page_token is not None and ais_ai.CURR_BUTTON_CODE == 22:
                        text = "Pobrano następną stronę wyników, włączam pierwszy: %s" % (list_info[1]["title"])
                        yield from self.hass.services.async_call('ais_yt_service', 'select_track_uri', {"id": 1})
                    elif prev_page_token is not None and ais_ai.CURR_BUTTON_CODE == 21:
                        text = "Pobrano poprzednią stronę wyników, włączam ostatni: %s" \
                               % (list_info[len(list_info) - 2]["title"])
                        yield from self.hass.services.async_call(
                            'ais_yt_service', 'select_track_uri', {"id": len(list_info) - 2})

        else:
            text = "Brak wnyników na YouTube dla zapytania %s" % query
        # info to user
        if text is not None:
            yield from self.hass.services.async_call('ais_ai_service', 'say_it', {"text": text})

    def process_select_track_uri(self, call):
        _LOGGER.info("process_select_track_uri")
        # """play track by id on sensor list."""
        call_id = call.data["id"]
        list_sensor = 'sensor.youtubelist'
        media_source = ais_global.G_AN_MUSIC
        if "media_source" in call.data:
            media_source = call.data["media_source"]
            if media_source == ais_global.G_AN_FAVORITE:
                list_sensor = 'sensor.aisfavoriteslist'
            elif media_source == ais_global.G_AN_BOOKMARK:
                list_sensor = 'sensor.aisbookmarkslist'

        state = self.hass.states.get(list_sensor)
        attr = state.attributes
        track = attr.get(int(call_id))
        media_position_ms = track.get("media_position_ms", 0)
        url = "https://www.youtube.com/watch?v="
        # update list
        self.hass.states.async_set(list_sensor, call_id, attr)

        # get prev / next results
        if track["uri"].startswith('prevPageToken_'):
            self.hass.services.call(
                'ais_yt_service', 'search', {"prevPageToken": track["uri"], "query": track["query"]})
            return
        if track["uri"].startswith('nextPageToken_'):
            self.hass.services.call(
                'ais_yt_service', 'search', {"nextPageToken": track["uri"], "query": track["query"]})
            return

        # try to get media url from AIS cloud
        media_url = None
        try:
            ws_resp = aisCloud.extract_media(url + track["uri"])
            json_ws_resp = ws_resp.json()
            _LOGGER.info(str(json_ws_resp))
            media_url = json_ws_resp["url"]
        except Exception as e:
            _LOGGER.error("extract_media error: " + str(e))

        all_ok = False
        if media_url is not None and len(media_url) > 0:
            # check 403
            import requests
            try:
                r = requests.head(media_url, allow_redirects=True, timeout=1)
                if r.status_code < 400:
                    all_ok = True
            except Exception as e:
                _LOGGER.warning("Request to youtube error " + str(e))

        if all_ok:
            # set stream url, image and title
            _audio_info = json.dumps(
                {"IMAGE_URL": track["thumbnail"], "NAME": track["title"], "lookup_url": track["uri"],
                 "MEDIA_SOURCE": media_source, "media_content_id": media_url, "media_position_ms": media_position_ms})
            self.hass.services.call(
                'media_player',
                'play_media', {
                    "entity_id": ais_global.G_LOCAL_EXO_PLAYER_ENTITY_ID,
                    "media_content_type": "ais_content_info",
                    "media_content_id": _audio_info
                })
        else:
            # use media_extractor to extract locally
            self.hass.services.call(
                'media_extractor',
                'play_media', {
                    "entity_id": ais_global.G_LOCAL_EXO_PLAYER_ENTITY_ID,
                    "media_content_id": url + track["uri"],
                    "media_content_type": "video/youtube"})

            # set stream image and title
            _audio_info = json.dumps( {"IMAGE_URL": track["thumbnail"], "NAME": track["title"],
                                       "MEDIA_SOURCE": ais_global.G_AN_MUSIC, "lookup_url": track["uri"]})
            self.hass.services.call('media_player', 'play_media',
                                    {"entity_id": ais_global.G_LOCAL_EXO_PLAYER_ENTITY_ID,
                                     "media_content_type": "ais_info",
                                     "media_content_id": _audio_info})



"""Support for Hydrawise sprinkler binary sensors."""
import logging

import voluptuous as vol

from homeassistant.components.binary_sensor import PLATFORM_SCHEMA, BinarySensorDevice
from homeassistant.const import CONF_MONITORED_CONDITIONS
import homeassistant.helpers.config_validation as cv

from . import (
    BINARY_SENSORS,
    DATA_HYDRAWISE,
    DEVICE_MAP,
    DEVICE_MAP_INDEX,
    HydrawiseEntity,
)

_LOGGER = logging.getLogger(__name__)

PLATFORM_SCHEMA = PLATFORM_SCHEMA.extend(
    {
        vol.Optional(CONF_MONITORED_CONDITIONS, default=BINARY_SENSORS): vol.All(
            cv.ensure_list, [vol.In(BINARY_SENSORS)]
        )
    }
)


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up a sensor for a Hydrawise device."""
    hydrawise = hass.data[DATA_HYDRAWISE].data

    sensors = []
    for sensor_type in config.get(CONF_MONITORED_CONDITIONS):
        if sensor_type in ["status", "rain_sensor"]:
            sensors.append(
                HydrawiseBinarySensor(hydrawise.controller_status, sensor_type)
            )

        else:
            # create a sensor for each zone
            for zone in hydrawise.relays:
                zone_data = zone
                zone_data["running"] = hydrawise.controller_status.get("running", False)
                sensors.append(HydrawiseBinarySensor(zone_data, sensor_type))

    add_entities(sensors, True)


class HydrawiseBinarySensor(HydrawiseEntity, BinarySensorDevice):
    """A sensor implementation for Hydrawise device."""

    @property
    def is_on(self):
        """Return true if the binary sensor is on."""
        return self._state

    def update(self):
        """Get the latest data and updates the state."""
        _LOGGER.debug("Updating Hydrawise binary sensor: %s", self._name)
        mydata = self.hass.data[DATA_HYDRAWISE].data
        if self._sensor_type == "status":
            self._state = mydata.status == "All good!"
        elif self._sensor_type == "rain_sensor":
            for sensor in mydata.sensors:
                if sensor["name"] == "Rain":
                    self._state = sensor["active"] == 1
        elif self._sensor_type == "is_watering":
            if not mydata.running:
                self._state = False
            elif int(mydata.running[0]["relay"]) == self.data["relay"]:
                self._state = True
            else:
                self._state = False

    @property
    def device_class(self):
        """Return the device class of the sensor type."""
        return DEVICE_MAP[self._sensor_type][
            DEVICE_MAP_INDEX.index("DEVICE_CLASS_INDEX")
        ]



"""Support for Abode Security System binary sensors."""
import logging

from homeassistant.components.binary_sensor import BinarySensorDevice

from . import DOMAIN as ABODE_DOMAIN, AbodeAutomation, AbodeDevice

_LOGGER = logging.getLogger(__name__)


def setup_platform(hass, config, add_entities, discovery_info=None):
    """Set up a sensor for an Abode device."""
    import abodepy.helpers.constants as CONST
    import abodepy.helpers.timeline as TIMELINE

    data = hass.data[ABODE_DOMAIN]

    device_types = [
        CONST.TYPE_CONNECTIVITY,
        CONST.TYPE_MOISTURE,
        CONST.TYPE_MOTION,
        CONST.TYPE_OCCUPANCY,
        CONST.TYPE_OPENING,
    ]

    devices = []
    for device in data.abode.get_devices(generic_type=device_types):
        if data.is_excluded(device):
            continue

        devices.append(AbodeBinarySensor(data, device))

    for automation in data.abode.get_automations(generic_type=CONST.TYPE_QUICK_ACTION):
        if data.is_automation_excluded(automation):
            continue

        devices.append(
            AbodeQuickActionBinarySensor(
                data, automation, TIMELINE.AUTOMATION_EDIT_GROUP
            )
        )

    data.devices.extend(devices)

    add_entities(devices)


class AbodeBinarySensor(AbodeDevice, BinarySensorDevice):
    """A binary sensor implementation for Abode device."""

    @property
    def is_on(self):
        """Return True if the binary sensor is on."""
        return self._device.is_on

    @property
    def device_class(self):
        """Return the class of the binary sensor."""
        return self._device.generic_type


class AbodeQuickActionBinarySensor(AbodeAutomation, BinarySensorDevice):
    """A binary sensor implementation for Abode quick action automations."""

    def trigger(self):
        """Trigger a quick automation."""
        self._automation.trigger()

    @property
    def is_on(self):
        """Return True if the binary sensor is on."""
        return self._automation.is_active



"""Component to manage a shopping list."""
import asyncio
import logging
import uuid
import json
from homeassistant.core import callback
from homeassistant.helpers import intent
from homeassistant.util.json import load_json, save_json
import homeassistant.ais_dom.ais_global as ais_global

DOMAIN = 'ais_bookmarks'
DEPENDENCIES = ['http']
_LOGGER = logging.getLogger(__name__)
INTENT_ADD_FAVORITE = 'AisBookmarksAddFavorite'
INTENT_ADD_BOOKMARK = 'AisBookmarksAddBookmark'
INTENT_LAST_BOOKMARKS = 'AisBookmarksLastBookmarks'
INTENT_PLAY_LAST_BOOKMARK = 'AisBookmarkPlayLastBookmark'
PERSISTENCE_BOOKMARKS = '.dom/.ais_bookmarks.json'
PERSISTENCE_FAVORITES = '.dom/.ais_favorites.json'

SERVICE_ADD_BOOKMARK = 'add_bookmark'
SERVICE_ADD_FAVORITE = 'add_favorite'
SERVICE_GET_BOOKMARKS = 'get_bookmarks'
SERVICE_GET_FAVORITES = 'get_favorites'
SERVICE_PLAY_BOOKMARK = 'play_bookmark'
SERVICE_PLAY_FAVORITE = 'play_favorite'
SERVICE_DELETE_BOOKMARK = 'delete_bookmark'
SERVICE_DELETE_FAVORITE = 'delete_favorite'


@asyncio.coroutine
def async_setup(hass, config):
    """Initialize the ais bookmarks list."""
    @asyncio.coroutine
    def add_bookmark_service(call):
        """Add an current played item to bookmark"""
        d = hass.data[DOMAIN]
        d.async_add(call, True)

    @asyncio.coroutine
    def add_favorite_service(call):
        """Add an current played item to favorite."""
        d = hass.data[DOMAIN]
        d.async_add(call, False)

    @asyncio.coroutine
    def get_bookmarks_service(call):
        """Return the list of bookmarks to app list"""
        d = hass.data[DOMAIN]
        list_info = {}
        list_idx = 0
        for item in reversed(d.bookmarks):
            if "media_stream_image" in item and item["media_stream_image"] is not None:
                img = item["media_stream_image"]
            else:
                img = "/static/icons/tile-win-310x150.png"
            list_info[list_idx] = {}
            list_info[list_idx]["title"] = item['name']
            if item['name'].startswith(item['source']):
                list_info[list_idx]["name"] = item['name']
            else:
                list_info[list_idx]["name"] =\
                    ais_global.G_NAME_FOR_AUDIO_NATURE.get(item['source'], item['source']) + " " + item['name']
            list_info[list_idx]["thumbnail"] = img
            list_info[list_idx]["uri"] = item["media_content_id"]
            list_info[list_idx]["audio_type"] = item['source']
            list_info[list_idx]["media_source"] = item['source']
            list_info[list_idx]["icon"] = 'mdi:bookmark-music'
            list_info[list_idx]["icon_remove"] = 'mdi:delete-forever'
            list_info[list_idx]["id"] = item['id']
            list_info[list_idx]["media_position"] = ais_global.get_milliseconds_formated(item['media_position'])
            list_info[list_idx]["media_position_ms"] = item['media_position']
            list_idx = list_idx + 1
        # create lists
        hass.states.async_set("sensor.aisbookmarkslist", -1, list_info)

    @asyncio.coroutine
    def get_favorites_service(call):
        """Return the list of favorites to app list"""
        audio_source = None
        if "audio_source" in call.data:
            audio_source = call.data["audio_source"]

        d = hass.data[DOMAIN]
        list_info = {}
        list_idx = 0
        for item in reversed(d.favorites):
            if audio_source is None or audio_source == item['source']:
                if "media_stream_image" in item and item["media_stream_image"] is not None:
                    img = item["media_stream_image"]
                else:
                    img = "/static/icons/tile-win-310x150.png"
                list_info[list_idx] = {}
                list_info[list_idx]["title"] = item['name']
                if item['name'].startswith(item['source']):
                    list_info[list_idx]["name"] = item['name']
                else:
                    list_info[list_idx]["name"] =\
                        ais_global.G_NAME_FOR_AUDIO_NATURE.get(item['source'], item['source']) + " " + item['name']
                list_info[list_idx]["thumbnail"] = img
                list_info[list_idx]["uri"] = item["media_content_id"]
                list_info[list_idx]["audio_type"] = item['source']
                list_info[list_idx]["icon_type"] = ais_global.G_ICON_FOR_AUDIO.get(item['source'], 'mdi:play')
                list_info[list_idx]["icon_remove"] = 'mdi:delete-forever'
                if audio_source == ais_global.G_AN_PODCAST:
                    list_info[list_idx]["icon"] = 'mdi:podcast'
                else:
                    list_info[list_idx]["icon"] = 'mdi:play'
                list_info[list_idx]["id"] = item['id']
                list_idx = list_idx + 1

        # create lists
        if audio_source is None:
            # get all items
            hass.states.async_set("sensor.aisfavoriteslist", -1, list_info)
        else:
            # check if the change was done form remote
            import homeassistant.components.ais_ai_service as ais_ai
            if audio_source == ais_global.G_AN_RADIO:
                hass.states.async_set("sensor.radiolist", -1, list_info)
                if ais_ai.CURR_ENTITIE == 'input_select.radio_type' and ais_ai.CURR_BUTTON_CODE == 23:
                    ais_ai.set_curr_entity(hass, 'sensor.radiolist')
                    hass.async_add_job(hass.services.async_call('ais_ai_service', 'say_it', {"text": "Wybierz stację"}))
            elif audio_source == ais_global.G_AN_PODCAST:
                hass.states.async_set("sensor.podcastnamelist", -1, list_info)
                if ais_ai.CURR_ENTITIE == 'input_select.podcast_type' and ais_ai.CURR_BUTTON_CODE == 23:
                    ais_ai.set_curr_entity(hass, 'sensor.podcastnamelist')
                    hass.async_add_job(hass.services.async_call('ais_ai_service', 'say_it', {"text": "Wybierz audycję"}))
            elif audio_source == ais_global.G_AN_MUSIC:
                hass.states.async_set("sensor.youtubelist", -1, list_info)
            elif audio_source == ais_global.G_AN_SPOTIFY:
                hass.states.async_set("sensor.spotifylist", -1, list_info)
            elif audio_source == ais_global.G_AN_AUDIOBOOK:
                hass.states.async_set("sensor.audiobookschapterslist", -1, list_info)


    @asyncio.coroutine
    def play_bookmark_service(call):
        """Play selected bookmark"""
        bookmark_id = int(call.data.get("id"))
        # get item from list
        state = hass.states.get("sensor.aisbookmarkslist")
        attr = state.attributes
        track = attr.get(bookmark_id)

        #
        if track['media_source'] == ais_global.G_AN_LOCAL:
            last_slash = track["uri"].rfind("/")
            if last_slash > 0:
                dir_path = track["uri"][0:last_slash]
            else:
                dir_path = track["uri"]
            hass.async_add_job(hass.services.async_call(
                'ais_drives_service', 'browse_path',
                {"path": dir_path, "file_path": track["uri"], "seek_position": track["media_position_ms"]}))

        elif track['media_source'] == ais_global.G_AN_MUSIC:
            hass.async_add_job(
                hass.services.async_call(
                    'ais_yt_service', 'select_track_uri', {"id": bookmark_id, "media_source": ais_global.G_AN_BOOKMARK})
            )
        else:
            _audio_info = json.dumps({"IMAGE_URL": track["thumbnail"], "NAME": track['title'],
                                      "audio_type": track['audio_type'],
                                      "MEDIA_SOURCE": track['media_source'],
                                      "media_content_id": track['uri'],
                                      "media_position_ms": track["media_position_ms"]})
            # set stream uri, image and title
            hass.async_add_job(
                hass.services.async_call(
                    'media_player',
                    'play_media', {
                        "entity_id": ais_global.G_LOCAL_EXO_PLAYER_ENTITY_ID,
                        "media_content_type": "ais_content_info",
                        "media_content_id": _audio_info
                    })
            )

    @asyncio.coroutine
    def play_favorite_service(call):
        """Play selected favorite"""
        favorite_id = int(call.data.get("id"))
        # get item from list
        state = hass.states.get("sensor.aisfavoriteslist")
        attr = state.attributes
        track = attr.get(favorite_id)

        _audio_info = json.dumps({"IMAGE_URL": track["thumbnail"], "NAME": track['title'],
                                  "audio_type": track['audio_type'],
                                  "MEDIA_SOURCE": ais_global.G_AN_FAVORITE, "media_content_id": track['uri']})
        # set stream uri, image and title
        hass.async_add_job(
            hass.services.async_call(
                'media_player',
                'play_media', {
                    "entity_id": ais_global.G_LOCAL_EXO_PLAYER_ENTITY_ID,
                    "media_content_type": "ais_content_info",
                    "media_content_id": _audio_info
                })
        )

    @asyncio.coroutine
    def delete_bookmark_service(call):
        """Delete selected bookmark"""
        bookmark_id = int(call.data.get("id"))
        state = hass.states.get('sensor.aisbookmarkslist')
        attr = state.attributes
        track = attr.get(bookmark_id)
        d = hass.data[DOMAIN]
        d.async_remove_bookmark(track['id'], True)

    @asyncio.coroutine
    def delete_favorite_service(call):
        """Delete selected favorite"""
        favorite_id = int(call.data.get("id"))
        state = hass.states.get('sensor.aisfavoriteslist')
        attr = state.attributes
        track = attr.get(favorite_id)
        d = hass.data[DOMAIN]
        d.async_remove_bookmark(track['id'], False)

    data = hass.data[DOMAIN] = BookmarksData(hass)
    intent.async_register(hass, AddFavoriteIntent())
    intent.async_register(hass, AddBookmarkIntent())
    intent.async_register(hass, ListTopBookmarkIntent())
    intent.async_register(hass, PlayLastBookmarkIntent())
    hass.services.async_register(DOMAIN, SERVICE_ADD_BOOKMARK, add_bookmark_service)
    hass.services.async_register(DOMAIN, SERVICE_ADD_FAVORITE, add_favorite_service)
    hass.services.async_register(DOMAIN, SERVICE_GET_BOOKMARKS, get_bookmarks_service)
    hass.services.async_register(DOMAIN, SERVICE_GET_FAVORITES, get_favorites_service)
    hass.services.async_register(DOMAIN, SERVICE_PLAY_BOOKMARK, play_bookmark_service)
    hass.services.async_register(DOMAIN, SERVICE_PLAY_FAVORITE, play_favorite_service)
    hass.services.async_register(DOMAIN, SERVICE_DELETE_BOOKMARK, delete_bookmark_service)
    hass.services.async_register(DOMAIN, SERVICE_DELETE_FAVORITE, delete_favorite_service)

    hass.components.conversation.async_register(INTENT_ADD_FAVORITE, [
        'Dodaj do ulubionych',
        'Do ulubionych',
        'Lubię to',
    ])
    hass.components.conversation.async_register(INTENT_ADD_BOOKMARK, ['Dodaj zakładkę', 'Zakładka'])
    hass.components.conversation.async_register(INTENT_LAST_BOOKMARKS, [
        'Jakie mam zakładki', 'Jakie są zakładki'
    ])
    hass.components.conversation.async_register(INTENT_PLAY_LAST_BOOKMARK, [
        'Włącz ostatnią zakładkę', 'Włącz zakładkę', 'Ostatnia zakładka'
    ])

    yield from data.async_load()

    hass.states.async_set("sensor.aisbookmarkslist", -1, {})
    hass.states.async_set("sensor.aisfavoriteslist", -1, {})

    return True


class BookmarksData:
    """Class to hold bookmarks list data."""

    def __init__(self, hass):
        """Initialize the bookmarks list."""
        self.hass = hass
        self.bookmarks = []
        self.favorites = []

    @callback
    def async_add(self, call, bookmark):
        """Add a item."""
        voice_call = False
        if 'voice_call' in call.data:
            voice_call = True

        if bookmark and "attr" not in call.data:
            # ask the player to add bookmark
            self.hass.async_add_job(
                self.hass.services.async_call(
                    'ais_ai_service', 'publish_command_to_frame', {"key": 'addBookmark', "val": True})
            )
            self.hass.async_add_job(
                self.hass.services.async_call(
                    'ais_ai_service', 'say_it', {"text": "Zakładka"})
            )
            return

        attributes = {}
        if "attr" in call.data:
            attributes = call.data["attr"]
            name = attributes.get("media_title").strip()
            source = attributes.get("source").strip()
            media_position = attributes.get("media_position", 0)
            media_content_id = attributes.get("media_content_id")
        else:
            state = self.hass.states.get(ais_global.G_LOCAL_EXO_PLAYER_ENTITY_ID)
            attributes = state.attributes
            name = attributes.get("media_title")
            source = attributes.get("source")
            media_position = attributes.get("media_position", 0)
            media_content_id = attributes.get("media_content_id")
        try:
            media_stream_image = ais_global.G_CURR_MEDIA_CONTENT["IMAGE_URL"]
        except Exception:
            media_stream_image = "/static/icons/favicon-100x100.png"

        try:
            audio_type = ais_global.G_CURR_MEDIA_CONTENT["audio_type"]
        except Exception:
            audio_type = source
        audio_type_pl = ais_global.G_NAME_FOR_AUDIO_NATURE.get(audio_type, audio_type)

        if name is None or source is None or media_content_id is None:
            _LOGGER.warning("can't add the bookmark, no full info provided " + str(attributes))
            return

        if bookmark:
            # type validation
            if audio_type == ais_global.G_AN_RADIO:
                message = "Nie można dodać zakładki do {}".format(audio_type_pl, name)
                self.hass.async_add_job(self.hass.services.async_call('ais_ai_service', 'say_it', {"text": message}))
                return

            #
            full_name = name
            if audio_type == ais_global.G_AN_LOCAL:
                media_content_id = ais_global.G_CURR_MEDIA_CONTENT["lookup_url"]
                full_name = ais_global.G_CURR_MEDIA_CONTENT["ALBUM_NAME"] + " " + name
            # if yt then not bookmark to current search but to lookup url
            elif audio_type == ais_global.G_AN_MUSIC:
                media_content_id = ais_global.G_CURR_MEDIA_CONTENT["lookup_url"]

            # check if the audio is on bookmark list
            item = next((itm for itm in self.bookmarks if (itm['media_content_id'] == media_content_id)), None)
            if item is not None:
                # delete the old bookmark
                self.async_remove_bookmark(item['id'], True)
                message = "Przesuwam zakładkę {}".format(full_name)
            else:
                message = "Dodaję nową zakładkę {}".format(full_name)

            # add the bookmark
            item = {
                'name': full_name,
                'id': uuid.uuid4().hex,
                'source': audio_type,
                'media_position': media_position,
                'media_content_id': media_content_id,
                'media_stream_image': media_stream_image
            }
            self.bookmarks.append(item)
            self.hass.async_add_job(self.save, True)
            if voice_call:
                self.hass.async_add_job(self.hass.services.async_call('ais_ai_service', 'say_it', {"text": message}))
            else:
                _LOGGER.info(message)
            return item
        else:
            # validation
            if source == ais_global.G_AN_FAVORITE:
                message = "{}, {} jest już w ulubionych.".format(audio_type_pl, name)
                self.hass.async_add_job(self.hass.services.async_call('ais_ai_service', 'say_it', {"text": message}))
                return
            # if podcast then we will add not track but audition
            if audio_type == ais_global.G_AN_PODCAST:
                media_content_id = ais_global.G_CURR_MEDIA_CONTENT["lookup_url"]
                name = ais_global.G_CURR_MEDIA_CONTENT["lookup_name"]
            elif audio_type == ais_global.G_AN_MUSIC:
                media_content_id = ais_global.G_CURR_MEDIA_CONTENT["lookup_url"]

            # check if the audio is on favorites list
            item = next((itm for itm in self.favorites if (itm['name'] == name and itm['source'] == audio_type)), None)
            if item is not None:
                message = '{}, {} jest już w ulubionych.'.format(audio_type_pl, name)
                self.hass.async_add_job(self.hass.services.async_call('ais_ai_service', 'say_it', {"text": message}))
                return
            # add item
            item = {
                'name': name,
                'id': uuid.uuid4().hex,
                'source': source,
                'media_content_id': media_content_id,
                'media_stream_image': media_stream_image
            }
            self.favorites.append(item)
            self.hass.async_add_job(self.save, False)
            message = "Dobrze zapamiętam - dodaje {} {} do Twoich ulubionych".format(audio_type_pl, name)
            self.hass.async_add_job(self.hass.services.async_call('ais_ai_service', 'say_it', {"text": message}))
            return item

    @callback
    def async_update(self, item_id, info):
        """Update a bookmarks list item."""
        item = next((itm for itm in self.bookmarks if itm['id'] == item_id), None)

        if item is None:
            raise KeyError

        item.update(info)
        self.hass.async_add_job(self.save)
        return item

    @callback
    def async_remove_bookmark(self, item_id, bookmark):
        if bookmark:
            """Reemove bookmark """
            self.bookmarks = [itm for itm in self.bookmarks if not itm['id'] == item_id]
            self.hass.async_add_job(self.save, True)
        else:
            """Reemove favorites """
            self.favorites = [itm for itm in self.favorites if not itm['id'] == item_id]
            self.hass.async_add_job(self.save, False)


    @asyncio.coroutine
    def async_load(self):
        """Load bookmarks."""
        def load():
            """Load the bookmarks synchronously."""
            try:
                self.bookmarks = load_json(self.hass.config.path(PERSISTENCE_BOOKMARKS), default=[])
                self.favorites = load_json(self.hass.config.path(PERSISTENCE_FAVORITES), default=[])
            except Exception as e:
                _LOGGER.error("Can't load bookmarks data: " + str(e))
        yield from self.hass.async_add_job(load)

    def save(self, bookmark):
        """Save the bookmarks and favorites."""
        if bookmark:
            self.bookmarks = self.bookmarks[-50:]
            save_json(self.hass.config.path(PERSISTENCE_BOOKMARKS), self.bookmarks)
            self.hass.async_add_job(self.hass.services.async_call(DOMAIN, SERVICE_GET_BOOKMARKS))
        else:
            self.favorites = self.favorites[-50:]
            save_json(self.hass.config.path(PERSISTENCE_FAVORITES), self.favorites)
            self.hass.async_add_job(self.hass.services.async_call(DOMAIN, SERVICE_GET_FAVORITES))


class AddFavoriteIntent(intent.IntentHandler):
    """Handle AddItem intents."""
    intent_type = INTENT_ADD_FAVORITE

    @asyncio.coroutine
    def async_handle(self, intent_obj):
        """Handle the intent."""
        hass = intent_obj.hass
        state = hass.states.get('media_player.wbudowany_glosnik')
        name = state.attributes.get("media_title")
        source = state.attributes.get("source")
        media_content_id = state.attributes.get("media_content_id")
        # check if all fields are provided
        if name is None or source is None or media_content_id is None:
            answer = "Nie można dodać do ulubionych - brak informacji o odtwarzanym audio."
        else:
            answer = "Dobrze zapamiętam - dodaje {} do Twoich ulubionych".format(name)
            # hass.data[DOMAIN].async_add(state.attributes, False)
            yield from hass.services.async_call('ais_bookmarks', 'add_favorite', {'voice_call': True})
        response = intent_obj.create_response()
        response.async_set_speech(answer)
        return response


class AddBookmarkIntent(intent.IntentHandler):
    """Handle AddItem intents."""
    intent_type = INTENT_ADD_BOOKMARK

    @asyncio.coroutine
    def async_handle(self, intent_obj):
        """Handle the intent."""
        hass = intent_obj.hass
        state = hass.states.get('media_player.wbudowany_glosnik')
        name = state.attributes.get("media_title")
        source = state.attributes.get("source")
        media_content_id = state.attributes.get("media_content_id")
        # check if all fields are provided
        if name is None or source is None or media_content_id is None:
            answer = "Nie można dodać zakładki - brak informacji o odtwarzanym audio."
        else:
            answer = "Dobrze dodaję zakładkę {}".format(name)
            yield from hass.services.async_call('ais_bookmarks', 'add_bookmark', {'voice_call': True})
        response = intent_obj.create_response()
        response.async_set_speech(answer)
        return response


class ListTopBookmarkIntent(intent.IntentHandler):
    """Handle ListTopBookmarkIntent intents."""

    intent_type = INTENT_LAST_BOOKMARKS

    @asyncio.coroutine
    def async_handle(self, intent_obj):
        """Handle the intent."""
        hass = intent_obj.hass
        bookmarks = intent_obj.hass.data[DOMAIN].bookmarks[-5:]
        response = intent_obj.create_response()
        if not bookmarks:
            answer = "Nie ma żadnych zakładek"
        else:
            answer = "Oto ostatnie {} zakładki na twojej liście: {}".format(
                        min(len(bookmarks), 5),
                        ', '.join(itm['name'] for itm in reversed(bookmarks)))
        response.async_set_speech(answer)
        yield from hass.services.async_call('ais_ai_service', 'say_it', {"text": answer})
        return response


class PlayLastBookmarkIntent(intent.IntentHandler):
    """Handle PlayLastBookmarkIntent intents."""

    intent_type = INTENT_PLAY_LAST_BOOKMARK

    @asyncio.coroutine
    def async_handle(self, intent_obj):
        """Handle the intent."""
        hass = intent_obj.hass
        bookmarks = intent_obj.hass.data[DOMAIN].bookmarks[-1:]
        response = intent_obj.create_response()
        if not bookmarks:
            answer = "Nie ma żadnych zakładek"
        else:
            bookmark = bookmarks[0]["source"] + '; ' + bookmarks[0]["name"]
            answer = "Włączam ostatnią zakładkę {}".format(bookmark)
            yield from hass.services.async_call('ais_bookmarks', 'play_bookmark', {"bookmark": bookmark})
        response.async_set_speech(answer)
        yield from hass.services.async_call('ais_ai_service', 'say_it', {"text": answer})
        return response
